<?php

/**
 * @file
 * Post update functions for Slots.
 */

use Drupal\Core\Entity\Sql\SqlEntityStorageInterface;

/**
 * Implements hook_post_update_NAME().
 *
 * Adds a status column to all existing slots fields.
 */
function slots_post_update_add_slots_columns_to_field(&$sandbox) {
  $definition_update_manager = \Drupal::entityDefinitionUpdateManager();
  /** @var \Drupal\Core\Entity\EntityLastInstalledSchemaRepositoryInterface $last_installed_schema_repository */
  $last_installed_schema_repository = \Drupal::service('entity.last_installed_schema.repository');

  /** @var \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager */
  $entity_field_manager = \Drupal::service('entity_field.manager');
  $slot_fields = $entity_field_manager->getFieldMapByFieldType('slots');

  foreach ($slot_fields as $entity_type_id => $fields) {
    $field_storage_definitions = $last_installed_schema_repository->getLastInstalledFieldStorageDefinitions($entity_type_id);
    $entity_type = $definition_update_manager->getEntityType($entity_type_id);

    // Update schema.
    $definition_update_manager->updateFieldableEntityType($entity_type, $field_storage_definitions, $sandbox);

    // Set default values.
    $storage = \Drupal::entityTypeManager()->getStorage($entity_type_id);
    if (!$storage instanceof SqlEntityStorageInterface) {
      continue;
    }
    $mapping = $storage->getTableMapping();
    foreach ($fields as $field_name => $definition) {
      foreach ($mapping->getTableNames() as $table_name) {
        if (in_array($field_name, $mapping->getFieldNames($table_name))) {
          \Drupal::database()->update($table_name)
            ->fields([
              $field_name . '_slots_status' => 1,
              $field_name . '_status' => 1,
            ])
            ->execute();
        }
      }
    }
  }

  return t('Slots fields have been updated.');
}
